//////////////////////////////////////////////////////////////////////////
// Creates C++ MEX-file for OpenCV routine matchTemplate. 
// Here matchTemplate uses normalized cross correlation method to search 
// for matches between an image patch and an input image.
//
// Copyright 2014 The MathWorks, Inc.
//////////////////////////////////////////////////////////////////////////

#include "opencvmex.hpp"

#define _DO_NOT_EXPORT
#if defined(_DO_NOT_EXPORT)
#define DllExport  
#else
#define DllExport __declspec(dllexport)
#endif

using namespace std;
using namespace cv;

///////////////////////////////////////////////////////////////////////////
// Check inputs
///////////////////////////////////////////////////////////////////////////
void checkInputs(int nrhs, const mxArray *prhs[])
{        
    // Check number of inputs
    if (nrhs != 5)
    {
        mexErrMsgTxt("Incorrect number of inputs. Function expects 5 inputs.");
    }

    // find_circles_ocv(src, radius, min_dist, edge_thresh, accum_thresh)    
    // src: source image
    if (mxGetNumberOfDimensions(prhs[0])>2 || !mxIsUint8(prhs[0]) || mxIsComplex(prhs[0]))
    {
        mexErrMsgTxt("src image must be a uint8 real matrix.");
    }
    // radius: radius range, 2 elements, double array
    if(mxGetNumberOfElements(prhs[1]) != 2 || !mxIsDouble(prhs[1]) || mxIsComplex(prhs[1]))
    {
        mexErrMsgTxt("radius range must be a real, 2 element double vector");
    }
    // min_dist: minimum distance between circles, scalar double
    if(!mxIsScalar(prhs[2]) || !mxIsDouble(prhs[2]) || mxIsComplex(prhs[2]))
    {
        mexErrMsgTxt("min_dist must be a scalar double");
    }
    // edge_thresh: threshold for canny edge detection
    if(!mxIsScalar(prhs[3]) || !mxIsDouble(prhs[3]) || mxIsComplex(prhs[3]))
    {
        mexErrMsgTxt("edge_thresh must be a scalar double");
    }
	// accum_thresh: accumulator vote threshold
    if(!mxIsScalar(prhs[4]) || !mxIsDouble(prhs[4]) || mxIsComplex(prhs[4]))
    {
        mexErrMsgTxt("accum_thresh must be a scalar double");
    }
}

///////////////////////////////////////////////////////////////////////////
// find_circles_ocv(src, radius, min_dist, edge_thresh, accum_thresh)
//  wrapper around cv::HoughCircles to find circles in image using Hough
//  transform 
///////////////////////////////////////////////////////////////////////////
void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{  
    double d_min_dist;
    double d_edge_threshold;
    double d_accumulator_threshold;
    double *d_radius_range;
    double *d_coord, *d_radius;

	// Check inputs to mex function
    checkInputs(nrhs, prhs);

    // Convert mxArray inputs into OpenCV types
    Ptr<Mat> src            = ocvMxArrayToImage_uint8(prhs[0], true);
    d_radius_range          = mxGetPr(prhs[1]);
    d_min_dist              = mxGetScalar(prhs[2]);
    d_edge_threshold        = mxGetScalar(prhs[3]);
    d_accumulator_threshold = mxGetScalar(prhs[4]);    

    // hough transform
    vector<Vec3f> circles;
    HoughCircles(*src, circles, HOUGH_GRADIENT, 
        1, 
        d_min_dist, 
        d_edge_threshold, 
        d_accumulator_threshold, 
        int(d_radius_range[0]), int(d_radius_range[1]) );
    
    // Put the data back into the output MATLAB array
    if(nlhs > 0)
    {
        plhs[0]     = mxCreateDoubleMatrix(2, circles.size(), mxREAL);
        plhs[1]     = mxCreateDoubleMatrix(1, circles.size(), mxREAL);
        d_coord     = mxGetPr(plhs[0]);
        d_radius    = mxGetPr(plhs[1]);
        for(size_t i = 0; i < circles.size(); ++i)
        {
            d_coord[i*2+0] = circles[i][0];   // x
            d_coord[i*2+1] = circles[i][1];   // y
            d_radius[i]    = circles[i][2];   // radius
        }
    }
}
